SystemOrganization addCategory: #HistoryCollection!

Object subclass: #HistoryCollection
	instanceVariableNames: 'storage maxSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HistoryCollection'!

!HistoryCollection commentStamp: 'BenjaminVanRyseghem 3/17/2011 13:41' prior: 0!
I'm a data structure used to store an history.

My behavior is
	- I store Associations (anElement -> anInteger), where anElement is the element to store and anInteger the number of times it has been added. (as a Dictionary)
	- I only store once each element (as a Set).
	- I have a max size. If I add an element and have reached the max size, I remove the less seen element.
	- I store element in the chronological order (as an OrderedCollection)!

!HistoryCollection class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 3/17/2011 13:47'!
maxSize: anInteger

	^ self new maxSize: anInteger! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:37'!
add: anElement

	| count |
	count := 0.
	(self includes: anElement)
		ifTrue: [ | association |
			association := self remove: anElement.
			count := association value ].
	self size = self maxSize
		ifTrue: [	self removeLast ].

	^ storage addFirst: (anElement -> (count+1))! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:43'!
at: anElement

	^ storage detect: [:each | each key = anElement]! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/18/2011 15:33'!
elements
	
	^ storage collect: [:each | each key ]! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:38'!
identityIncludes: anElement

	^ storage anySatisfy: [:association | association key == anElement ]! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:37'!
includes: anElement

	^ storage anySatisfy: [:association | association key = anElement ]! !

!HistoryCollection methodsFor: 'private' stamp: 'BenjaminVanRyseghem 3/17/2011 13:47'!
initialize

	super initialize.
	maxSize := 5.
	storage := OrderedCollection new: maxSize.! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 14:00'!
lessSeen

	| min |
	min := storage first.
	2 to: storage size do: [:index || each |
		each := storage at: index.
		(each value < min value)
			ifTrue: [ min := each]].
	^ min key! !

!HistoryCollection methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 3/17/2011 13:31'!
maxSize

	^ maxSize! !

!HistoryCollection methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 3/17/2011 13:32'!
maxSize: anInteger

	maxSize := anInteger! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 14:05'!
mostSeen

	| max |
	max := storage first.
	2 to: storage size do: [:index || each |
		each := storage at: index.
		(each value > max value)
			ifTrue: [ max := each]].
	^ max key! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 9/14/2012 15:48'!
mostViewedElements
	
	^ (storage copy sort: [:a :b | a value >= b value]) collect: [:each | each key ]! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 14:03'!
oldest

	^ storage last key! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:49'!
remove: anElement

	| association |
	association := self at: anElement.
	^ storage remove: association.
	! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:54'!
removeLast
	" I remove the less seen one, in case of equality, I remove the latest one "


	| min reversedStorage |
	reversedStorage := storage reverse.
	min := reversedStorage first.
	2 to: storage size do: [:index || each |
		each := reversedStorage at: index.
		(each value < min value)
			ifTrue: [ min := each ]].
	storage remove: min! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:49'!
size

	^ storage size! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 14:03'!
youngest

	^ storage first key! !

Object subclass: #NavigationHistory
	instanceVariableNames: 'storage index maxSize paused'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HistoryCollection'!

!NavigationHistory commentStamp: '<historical>' prior: 0!
I act as a web browser history!

!NavigationHistory class methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 3/13/2012 16:37'!
defaultMaxSize

	^ 15! !

!NavigationHistory methodsFor: 'collection compatibility' stamp: 'BenjaminVanRyseghem 3/13/2012 16:13'!
add: anEntry

	self addEntry: anEntry! !

!NavigationHistory methodsFor: 'protocol' stamp: 'CamilloBruni 10/4/2012 11:29'!
addEntry: anEntry
	
	self isPaused ifTrue: [ ^ self ].
	
	( storage isEmpty not  and: [ storage last = anEntry ] ) ifTrue: [ ^ self ].
	anEntry = self current key ifTrue: [ ^ self ].
	self checkSize.
	index := index +1.
	index > storage size
		ifTrue: [ storage addLast:  (anEntry -> TimeStamp now)]
		ifFalse: [ 
			storage := storage copyFrom: 1 to: index.
			storage at: index put: (anEntry -> TimeStamp now)]! !

!NavigationHistory methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/11/2011 15:09'!
back

	index := ((index - 1) max: 0)! !

!NavigationHistory methodsFor: 'private' stamp: 'BenjaminVanRyseghem 5/24/2011 13:42'!
checkSize
	
	[ storage size >= self maxSize ]
		whileTrue: [ 
			storage removeFirst.
			index := index -1].! !

!NavigationHistory methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/11/2011 16:11'!
current

	^ storage at: index ifAbsent: [ nil -> nil ]! !

!NavigationHistory methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/13/2012 04:03'!
entries

	^ storage! !

!NavigationHistory methodsFor: 'testing' stamp: 'CamilloBruni 10/4/2012 11:17'!
hasNext

	^ index < storage size! !

!NavigationHistory methodsFor: 'testing' stamp: 'BenjaminVanRyseghem 5/12/2011 13:01'!
hasPrevious

	^ index > 1! !

!NavigationHistory methodsFor: 'initialize-release' stamp: 'CamilloBruni 10/4/2012 11:29'!
initialize

	storage := OrderedCollection new.
	index := 0.
	paused := false.! !

!NavigationHistory methodsFor: 'testing' stamp: 'CamilloBruni 10/4/2012 11:30'!
isPaused
	^ paused! !

!NavigationHistory methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/11/2011 15:09'!
last

	^ storage last! !

!NavigationHistory methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/19/2011 11:05'!
maxSize

	^ maxSize ifNil: [ maxSize := self class defaultMaxSize ].! !

!NavigationHistory methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/14/2011 22:53'!
maxSize: anInteger

	maxSize := anInteger! !

!NavigationHistory methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 5/12/2011 12:25'!
next

	index := ((index + 1) min: storage size).
	^ storage at: index! !

!NavigationHistory methodsFor: 'protocol' stamp: 'CamilloBruni 10/4/2012 11:32'!
pauseDuring: aBlock
	| previousPausedValue |
	"Don't accept any new entries while executing aBlock"
	previousPausedValue := paused.
	paused := true.
	aBlock ensure: [ paused := previousPausedValue ]! !

!NavigationHistory methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 5/12/2011 12:25'!
previous

	index := index-1 max: 1.
	^ storage at: index! !
